class UnrealShare_Grenade expands Grenade;

var bool bCanCrossWaterSurface;

// replicated built-in vectors hold inexact values, float values are replicated precisely
struct ReplicatedVector
{
	var float X, Y, Z;
};

var ReplicatedVector ReplicatedLocation;
var ReplicatedVector ReplicatedVelocity;
var ReplicatedVector ReplicatedAcceleration;

var UGoldFixSharedInfo SharedInfo;

replication
{
	// Variables the server should send to the client.
	reliable if (Role == ROLE_Authority)
		ReplicatedLocation,
		ReplicatedVelocity,
		ReplicatedAcceleration;
}

simulated function PostBeginPlay()
{
	local vector X,Y,Z;
	local rotator RandRot;

	Super.PostBeginPlay();
	if (Level.NetMode != NM_DedicatedServer)
		PlayAnim('WingIn');
	if (Level.NetMode != NM_Client)
	{
		SetTimer(2.5 + FRand() * 0.5, false); // Grenade begins unarmed

		if (Instigator != none)
		{
			GetAxes(Instigator.ViewRotation, X, Y, Z);
			Velocity = X * (Instigator.Velocity Dot X) * 0.4 + Vector(Rotation) * (Speed + FRand() * 100);
		}
		else
			Velocity = Vector(Rotation) * (Speed + FRand() * 100);
		Velocity.Z += 210;
		RandRot.Pitch = FRand() * 1400 - 700;
		RandRot.Yaw = FRand() * 1400 - 700;
		RandRot.Roll = FRand() * 1400 - 700;
		MaxSpeed = 1000;
		Velocity = Velocity >> RandRot;
		RandSpin(50000);	
		bCanHitOwner = False;
		if (Region.Zone.bWaterZone)
		{
			bHitWater = True;
			Velocity = 0.6 * Velocity;			
		}
	}
	if (Level.NetMode != NM_Standalone)
		Server_UpdateMovement();
}

simulated function BeginPlay()
{
	if (Level.bHighDetailMode)
		SmokeRate = 0.03;
	else
		SmokeRate = 0.15;
}

simulated function ZoneChange( Zoneinfo NewZone )
{
	local WaterRing w;
	
	if (!bCanCrossWaterSurface || Region.Zone.bWaterZone == NewZone.bWaterZone)
		return;

	if (Level.NetMode != NM_DedicatedServer)
	{
		w = Spawn(class'WaterRing',,,,rot(16384,0,0));
		w.DrawScale = 0.2;
		w.RemoteRole = ROLE_None;
	}
	bHitWater = True;
	Velocity=0.6*Velocity;
}

function Timer()
{
	Explosion(Location+Vect(0,0,1)*16);
}

simulated function Tick(float DeltaTime)
{
	bCanCrossWaterSurface = true;

	if (Level.NetMode != NM_Standalone)
	{
		Server_UpdateMovement();
		Client_UpdateMovement();
	}

	if (Level.NetMode == NM_DedicatedServer)
		return;

	MakeGrenadeTrail(DeltaTime);
}

simulated function MakeGrenadeTrail(float DeltaTime)
{
	local BlackSmoke smoke;
	local float GrenadeTrailInterval;

	Count += DeltaTime;
	GrenadeTrailInterval = Frand() * SmokeRate + SmokeRate + NumExtraGrenades * 0.03;
	if (Count >= GrenadeTrailInterval)
	{
		if (!Region.Zone.bWaterZone)
		{
			smoke = Spawn(class'BlackSmoke');
			if (smoke != none)
				smoke.RemoteRole = ROLE_None;
		}
		Count -= GrenadeTrailInterval;
		if (Count > GrenadeTrailInterval)
			Count = GrenadeTrailInterval;
	}
}

function ProcessTouch(actor Other, vector HitLocation)
{
	if (Other != Instigator || bCanHitOwner)
		Explosion(HitLocation);
}

simulated function HitWall(vector HitNormal, actor Wall)
{
	bCanHitOwner = True;
	Velocity = 0.8*(( Velocity dot HitNormal ) * HitNormal * (-2.0) + Velocity);   // Reflect off Wall w/damping
	RandSpin(100000);
	speed = VSize(Velocity);
	if (Level.NetMode != NM_DedicatedServer)
		PlaySound(ImpactSound, SLOT_Misc, FMax(0.5, speed/800) );
	if (Velocity.Z > 400)
		Velocity.Z = 0.5 * (400 + Velocity.Z);	
	else if (speed < 20 && Level.NetMode != NM_Client) 
	{
		bBounce = False;
		SetPhysics(PHYS_None);
	}
}

function BlowUp(vector HitLocation)
{
	class'UGoldFix'.static.Projectile_HurtRadius(
		SharedInfo, self, damage, 200, 'exploded', MomentumTransfer, HitLocation);
	MakeNoise(1.0);
}

function Explosion(vector HitLocation)
{
	local SpriteBallExplosion explosion_effect;
	local UnrealShare_Grenade_Explosion explosion_info;

	BlowUp(HitLocation);

	if (Level.NetMode == NM_DedicatedServer)
	{
		// makes explosion sound on dedicated servers
		explosion_effect = Spawn(class'SpriteBallExplosion',,, HitLocation);
		explosion_effect.RemoteRole = ROLE_None;
	}

	explosion_info = Spawn(class'UnrealShare_Grenade_Explosion');
	explosion_info.AssignSourceLocation(Location);
	explosion_info.AssignHitLocation(HitLocation);
	explosion_info.SharedInfo = SharedInfo;

 	Destroy();
}

function Server_UpdateMovement()
{
	ReplicatedLocation = ConvertToReplicatedVector(Location);
	ReplicatedVelocity = ConvertToReplicatedVector(Velocity);
	ReplicatedAcceleration = ConvertToReplicatedVector(Acceleration);
}

simulated function Client_UpdateMovement()
{
	if (Level.NetMode == NM_Client &&
		VSize(ConvertReplicatedVector(ReplicatedLocation)) != 0)
	{
		SetLocation(ConvertReplicatedVector(ReplicatedLocation));
		Velocity = ConvertReplicatedVector(ReplicatedVelocity);
		Acceleration = ConvertReplicatedVector(ReplicatedAcceleration);
		ReplicatedLocation = ConvertToReplicatedVector(vect(0, 0, 0));
	}
}


static function ReplicatedVector ConvertToReplicatedVector(vector pos)
{
	local ReplicatedVector result;
	result.X = pos.X;
	result.Y = pos.Y;
	result.Z = pos.Z;
	return result;
}

static function vector ConvertReplicatedVector(ReplicatedVector pos)
{
	local vector result;
	result.X = pos.X;
	result.Y = pos.Y;
	result.Z = pos.Z;
	return result;
}

defaultproperties
{
				bNetTemporary=False
				bGameRelevant=False
}
